# Data

The Data tab allows users to explore the records associated with a given dataset. The tab includes two main components: 

1. a data table;
2. a map, displayed only for geographical datasets (it makes sense, doesn't it?).

![Data](../../../../assets/webapp/data.png)

The user can toggle the visibility of each of the two components by clicking on the dedicated buttons. The entire tab can also go fullscreen.

## Data table

Each entry of a given dataset is rendered as a line in a table. Data value having simple types (string, integer, float, ...) are fully displayed. Complex data values, such as JSON objects, are replaced by a placeholder signaling the presence of a complex property. Records are retrieved from the search engine in a paginated way (30 items per page), in order not to overload the browser.
To achieve this, we do not use the native scrolling feature of Elasticsearch, but the pagination. We retrieve records 30 by 30, changing the `from` property for each request.
A maximum limit of 10 000 records displayed has been set in the table.

### Search within the data entries

A dedicated text input allows the user to perform a full-text search within the entire dataset. The number of matching records is shown and visually illustrated in the guise of a progress bar.
To achieve this search, we concatenate each word in the input with the `&` character. This is done in order to have a very specific search, where each record contains all the terms.

### Column visibility

The visibility of each column can be toggled by clicking on a dedicated button and (de)selecting the various checkboxes.

### User interaction with the data table

* Each column can be sorted either alphabetically or numerically, according to the data type detected by the search engine.

* Upon clicking on a given entry on the table, the entry gets fully displayed (including complex properties) on a dedicated viewer. If the map component is visible, then the map gets centered on the clicked entry.

* Already visited entries are highlighted, on the table, with a different background color.

## Map

### Under the hood

The map component is powered by the [Mapbox GL JS](https://github.com/mapbox/mapbox-gl-js) library.

### Layers

Three overlaid layers are used:

1. a base layer, either vector or raster depending of the choice of the user. The raster layer is based on the [orthoimages](https://data.grandlyon.com/en/jeux-de-donnees/orthophotographie-2018-metropole-lyon-format-tiff/donnees) provided by the Métropole de Lyon with the addition of street labels coming from the vector base layer.

2. A raster layer, displaying the entire (geographic) dataset by issuing requests against a Web Map Service (WMS).

3. A transparent vector layer, fed by a Mapbox Vector Tiles (MVT) service, allowing users to interact with the raster features displayed by the WMS. Individual features get highlighted on hover and click events.

#### Restricted-access datasets

Some of the datasets hosted by data.grandlyon.com are only accessible to authorized users. For such datasets, two different behaviors were implemented:

1. for **authorized users**: the very same behavior as for unrestriced datasets, in a seamless way;

2. for **unauthorized users**: the raster layer, powered by the WMS, displays the full dataset; user interaction is limited to a small subset of geographical features and achieved by constructing a GeoJSON out of the sample returned by the search engine.

This feature relies on a custom user-aware component, proxying requests to the Web Mapping Services hosted by the data.grandlyon.com Core.

#### 3D emulation

3D emulation for buildings is achieved thanks to the *ad hoc* styling of the polygones returned by the following dataset: https://data.grandlyon.com/jeux-de-donnees/volumes-toiture-3d-2015-bati-metropole-lyon/donnees. More specifically, the `fill-extrusion` property is used.

### Search by address & geolocation

Users can search for addresses and points of interest by typing terms in the dedicated form. This feature relies on a geocoding API which is powered by [Photon](https://github.com/komoot/photon) and hosted by data.grandlyon.com.

Users can center the map on their current geolocation, as well. The feature relies on the native [Navigator API](https://developer.mozilla.org/en-US/docs/Web/API/Navigator/geolocation).

### *Share* feature

A button allows users to generate a share link, using the following pattern: `http://<hostName>#<zoom>/<latitude>/<longitude>/<bearing>/<pitch>/<selectedBaseLayerId>`. Such a link can be shared with other users in order to let the recipients land on the map with a given context: zoom, center, bearing, pitch and base layer.  

### User interaction with the map

Upon clicking on a geographical feature, the latter gets fully displayed (including complex properties) on a dedicated viewer. At the same, the feature is highlighted on the data table and pushed to the first row.

### Styling

The OpenMapTiles basemap [Klokantech Basic](https://openmaptiles.org/styles/#klokantech-basic) has been used as a starting point to generate data.grandlyon.com's custom styles. Such styles are compliant with the the [Mapbox GL style specification](https://docs.mapbox.com/mapbox-gl-js/style-spec/) and publicly available (cf. https://openmaptiles.data.grandlyon.com/).

#### Sprites

The map component uses custom sprites (*e.g.* the grass in green spaces), generated via the following workflow:

* download the [Mapbox icons](https://labs.mapbox.com/maki-icons/) (the Maki icons)
* add the custom SVG we want to use in our style (for example grass.svg and water.svg)
* clone and install [spritezero-cli](https://github.com/mapbox/spritezero-cli), a command-line interface for [spritezero](https://github.com/mapbox/spritezero)
* `spritezero [output filename] [input directory]` . It generates a JSON layout file and the associated PNG spritesheet.
Also generate it with the `--retina` option to have the @2x sprites
* Once the folder containing these files is hosted, we reference it in the style file:  
`"sprites": 'path/to/the/sprites/folder/'`
* Finally we can style the custom grass in green area:

```json
"paint": {
        "fill-pattern": "grass"
      }
```

#### Glyphs

The map component uses the [Titillium Web](https://fonts.google.com/specimen/Titillium+Web) font, the same as the rest of the Web application. The Mapbox GL JS library uses the notion of "glyphs" instead of "fonts". Glyphs are rendered as SDFs (Signed Distance Fields) and are better suited than fonts to rotation and scaling.

The [node-fontnik](https://github.com/mapbox/node-fontnik) tool can be used to convert `.ttf` files into protobuf-encoded SDF glyphs. Multiple fonts can be combined by using this other tool: [glyph-pbf-composite](https://github.com/mapbox/glyph-pbf-composite).

**TO ADD: "bannière rouge"**
